
########################################
#	Auteur : Pierre Aubert
#	Mail : aubertp7@gmail.com
#	Licence : CeCILL-C
########################################


# Set the CPU flags
# 	CPU_FLAG : will contain the CPU flags
function(plib_core_getAllFlag CPU_FLAG)
	set(OUTPUT_FLAG_STR )
	if(APPLE)
		message(STATUS "Get OSX extensions")
		execute_process(COMMAND bash "-c" "sysctl -a | grep machdep.cpu"
				WORKING_DIRECTORY ${CMAKE_CURRENT_BINARY_DIR}
				OUTPUT_VARIABLE OUTPUT_FLAG_STR
				)
	else()
		message(STATUS "Get LINUX extensions")
		execute_process(COMMAND cat /proc/cpuinfo
				WORKING_DIRECTORY ${CMAKE_CURRENT_BINARY_DIR}
				OUTPUT_VARIABLE OUTPUT_FLAG_STR
				)
	endif()
	string(TOLOWER ${OUTPUT_FLAG_STR} OUTPUT_FLAG_STR)
	set(${CPU_FLAG} ${OUTPUT_FLAG_STR} PARENT_SCOPE)
endfunction(plib_core_getAllFlag)

# Set the vectorization mode with the vectorization flags of the CPU
# 	VECT_MODE : vectorization mode (ssse3, sse4, avx, avx2, avx512)
function(plib_core_getVectModeWithFlag VECT_MODE)
	plib_core_getAllFlag(OUTPUT_FLAG_STR)
	set(TMP_VECT_MODE "none")
	string(REGEX MATCH "ssse3" matchSSSE3 ${OUTPUT_FLAG_STR})
	if(matchSSSE3)
		message(STATUS "Find SSSE3")
		set(TMP_VECT_MODE "ssse3")
	endif()

	string(REGEX MATCH "sse4" matchSSE4 ${OUTPUT_FLAG_STR})
	if(matchSSE4)
		message(STATUS "Find SSE4")
		set(TMP_VECT_MODE "sse4")
	endif()

	string(REGEX MATCH "avx" matchAVX ${OUTPUT_FLAG_STR})
	if(matchAVX)
		message(STATUS "Find AVX")
		set(TMP_VECT_MODE "avx")
	endif()

	string(REGEX MATCH "avx2" matchAVX2 ${OUTPUT_FLAG_STR})
	if(matchAVX2)
		message(STATUS "Find AVX2")
		set(TMP_VECT_MODE "avx2")
	endif()
	
	string(REGEX MATCH "avx512f" matchAVX512 ${OUTPUT_FLAG_STR})
	if(matchAVX512)
		message(STATUS "Find AVX512F")
		set(TMP_VECT_MODE "avx512f")
	endif()
	
	string(REGEX MATCH "avx512er" matchAVX5122 ${OUTPUT_FLAG_STR})
	if(matchAVX5122)
		message(STATUS "Find AVX512ER (Exponential and Reciprocal)")
		set(TMP_VECT_MODE "avx512er")
	endif()
	
	string(REGEX MATCH "avx512bw" matchAVX512BW ${OUTPUT_FLAG_STR})
	if(matchAVX5122)
		message(STATUS "Find AVX512BW (Bytes adnd Words intruction)")
		set(TMP_VECT_MODE "avx512bw")
	endif()
	
	string(REGEX MATCH "avx512.2" matchAVX5122 ${OUTPUT_FLAG_STR})
	if(matchAVX5122)
		message(STATUS "Find AVX512.2")
		set(TMP_VECT_MODE "avx512.2")
	endif()
	
	set(${VECT_MODE} ${TMP_VECT_MODE} PARENT_SCOPE)
endfunction(plib_core_getVectModeWithFlag)

# Set the vectorization mode
# 	VECT_MODE : vectorization mode (ssse3, sse4, avx, avx2, avx512)
function(plib_core_getVectMode VECT_MODE)
	if ( ${AVX512_2} )
		message(STATUS "Manual avx512.2 activated")
		set(${VECT_MODE}  "avx512f" PARENT_SCOPE)
	elseif ( ${AVX512VPOPCNTDQ} )
		message(STATUS "Manual avx512VPOPCNTDQ activated")
		set(${VECT_MODE}  "avx512vpopcntdq" PARENT_SCOPE)
	elseif ( ${AVX512BW} )
		message(STATUS "Manual avx512bw (Byte and Word instructon) activated")
		set(${VECT_MODE}  "avx512bw" PARENT_SCOPE)
	elseif ( ${AVX512PF} )
		message(STATUS "Manual avx512pf (PreFetch instructon) activated")
		set(${VECT_MODE}  "avx512pf" PARENT_SCOPE)
	elseif ( ${AVX512ER} )
		message(STATUS "Manual avx512er (Exponential and Reciprocal) activated")
		set(${VECT_MODE}  "avx512er" PARENT_SCOPE)
	elseif ( ${AVX512F} )
		message(STATUS "Manual avx512f (float) activated")
		set(${VECT_MODE}  "avx512f" PARENT_SCOPE)
	elseif ( ${AVX2} )
		message(STATUS "Manual avx2 activated")
		set(${VECT_MODE}  "avx2" PARENT_SCOPE)
	elseif ( ${AVX} )
		message(STATUS "Manual avx activated")
		set(${VECT_MODE}  "avx" PARENT_SCOPE)
	elseif ( ${SSE4.2} )
		message(STATUS "Manual sse4.2 activated")
		set(${VECT_MODE}  "sse4.2" PARENT_SCOPE)
	elseif ( ${SSE4.1} )
		message(STATUS "Manual sse4.1 activated")
		set(${VECT_MODE}  "sse4.1" PARENT_SCOPE)
	elseif ( ${SSSE3} )
		message(STATUS "Manual ssse3 activated")
		set(${VECT_MODE}  "ssse3" PARENT_SCOPE)
	elseif ( ${SSE2} )
		message(STATUS "Manual sse2 activated")
		set(${VECT_MODE}  "sse2" PARENT_SCOPE)
	elseif ( ${NOVECTORIZATION} )
		message(STATUS "Manual vectorization disabled")
		set(${VECT_MODE}  "none" PARENT_SCOPE)
	else()
		message(STATUS "Automatic searching for architecture flags")
		plib_core_getVectModeWithFlag(TMP_VECT_MODE)
		set(${VECT_MODE} ${TMP_VECT_MODE} PARENT_SCOPE)
	endif()
endfunction(plib_core_getVectMode)

# Set the vectorization mode
# 	VECT_MODE : vectorization mode (ssse3, sse4, avx, avx2, avx512)
function(plib_core_getVectModeStr VECT_MODE)
	if ( ${AVX512_2} )
		message(STATUS "Manual avx512.2 activated")
		set(${VECT_MODE}  "avx512f" PARENT_SCOPE)
	elseif ( ${AVX512VPOPCNTDQ} )
		message(STATUS "Manual avx512VPOPCNTDQ activated")
		set(${VECT_MODE}  "avx512vpopcntdq" PARENT_SCOPE)
	elseif ( ${AVX512BW} )
		message(STATUS "Manual avx512bw (Byte and Word instructon) activated")
		set(${VECT_MODE}  "avx512bw" PARENT_SCOPE)
	elseif ( ${AVX512PF} )
		message(STATUS "Manual avx512pf (PreFetch instructon) activated")
		set(${VECT_MODE}  "avx512pf" PARENT_SCOPE)
	elseif ( ${AVX512ER} )
		message(STATUS "Manual avx512er (Exponential and Reciprocal) activated")
		set(${VECT_MODE}  "avx512er" PARENT_SCOPE)
	elseif ( ${AVX512F} )
		message(STATUS "Manual avx512f (float) activated")
		set(${VECT_MODE}  "avx512f" PARENT_SCOPE)
	elseif ( ${AVX2} )
		message(STATUS "Manual avx2 activated")
		set(${VECT_MODE}  "avx2" PARENT_SCOPE)
	elseif ( ${AVX} )
		message(STATUS "Manual avx activated")
		set(${VECT_MODE}  "avx" PARENT_SCOPE)
	elseif ( ${SSE4.2} )
		message(STATUS "Manual sse4.2 activated")
		set(${VECT_MODE}  "sse42" PARENT_SCOPE)
	elseif ( ${SSE4.1} )
		message(STATUS "Manual sse4.1 activated")
		set(${VECT_MODE}  "sse41" PARENT_SCOPE)
	elseif ( ${SSSE3} )
		message(STATUS "Manual ssse3 activated")
		set(${VECT_MODE}  "ssse3" PARENT_SCOPE)
	elseif ( ${SSE2} )
		message(STATUS "Manual sse2 activated")
		set(${VECT_MODE}  "sse2" PARENT_SCOPE)
	elseif ( ${NOVECTORIZATION} )
		message(STATUS "Manual vectorization disabled")
		set(${VECT_MODE}  "anyarch" PARENT_SCOPE)
	else()
		message(STATUS "Automatic searching for architecture flags")
		plib_core_getVectModeWithFlag(TMP_VECT_MODE)
		set(${VECT_MODE} ${TMP_VECT_MODE} PARENT_SCOPE)
	endif()
endfunction(plib_core_getVectModeStr)


# Transform the input VECT_MODE into a MODE_NUNBER
# 	MODE_NUNBER : output node number 1 : ssse3, 2 : sse4.1, 3 : sse4.2, 4 : avx, 5 : avx2, 6 avx512.1, 7 avx512.2
# 	VECT_MODE : vectorial node
function(plib_core_getModeNumber MODE_NUNBER VECT_MODE)
	set(MODE_NUNBER 0 PARENT_SCOPE)
	if(VECT_MODE STREQUAL "ssse3")
		set(MODE_NUNBER 1 PARENT_SCOPE)
	endif()

	if(VECT_MODE STREQUAL "sse4.1")
		set(MODE_NUNBER 2 PARENT_SCOPE)
	endif()
	
	if(VECT_MODE STREQUAL "sse4.2")
		set(MODE_NUNBER 3 PARENT_SCOPE)
	endif()

	if(VECT_MODE STREQUAL "avx")
		set(MODE_NUNBER 4 PARENT_SCOPE)
	endif()

	if(VECT_MODE STREQUAL "avx2")
		set(MODE_NUNBER 5 PARENT_SCOPE)
	endif()
	
	if(VECT_MODE STREQUAL "avx512f")
		set(MODE_NUNBER 6 PARENT_SCOPE)
	endif()
	
	if(VECT_MODE STREQUAL "avx512er")
		set(MODE_NUNBER 7 PARENT_SCOPE)
	endif()
	
	if(VECT_MODE STREQUAL "avx512pf")
		set(MODE_NUNBER 8 PARENT_SCOPE)
	endif()
	
	if(VECT_MODE STREQUAL "avx512bw")
		set(MODE_NUNBER 9 PARENT_SCOPE)
	endif()
	
	if(VECT_MODE STREQUAL "avx512vpopcntdq")
		set(MODE_NUNBER 10 PARENT_SCOPE)
	endif()
	
	if(VECT_MODE STREQUAL "avx512.2")
		set(MODE_NUNBER 11 PARENT_SCOPE)
	endif()
	
	if(VECT_MODE STREQUAL "none")
		set(MODE_NUNBER 0 PARENT_SCOPE)
	endif()
endfunction(plib_core_getModeNumber)

# Create the include file with the different extensions
# 	outputFileName : name of the output file to be generated
function(plib_core_createDefInclude outputFileName)
	get_filename_component(baseFileName ${outputFileName} NAME)
	set(OUTPUT_FILE "${CMAKE_CURRENT_BINARY_DIR}/${baseFileName}")
	if(EXISTS ${OUTPUT_FILE})
		message(STATUS "File '${OUTPUT_FILE}' already exists")
	else()
		plib_core_getVectMode(VECT_MODE)
		string(REPLACE "." "_" lowDefine ${baseFileName})
		string(TOUPPER ${lowDefine} upperDefine)
		
		file(WRITE ${OUTPUT_FILE} "\n")
		file(APPEND ${OUTPUT_FILE} "/***************************************\n")
		file(APPEND ${OUTPUT_FILE} "	Auteur : Pierre Aubert\n")
		file(APPEND ${OUTPUT_FILE} "	Mail : aubertp7@gmail.com\n")
		file(APPEND ${OUTPUT_FILE} "	Licence : CeCILL-C\n")
		file(APPEND ${OUTPUT_FILE} "****************************************/\n")
		file(APPEND ${OUTPUT_FILE} "\n")
		file(APPEND ${OUTPUT_FILE} "#ifndef __${upperDefine}__\n")
		file(APPEND ${OUTPUT_FILE} "#define __${upperDefine}__\n")
		file(APPEND ${OUTPUT_FILE} "\n")
		file(APPEND ${OUTPUT_FILE} "\n#include <immintrin.h>\n\n")
		
		plib_core_getModeNumber(MODE_NUNBER ${VECT_MODE})
		
		message(STATUS "CMAKE_VERSION = ${CMAKE_VERSION}, MODE_NUNBER = '${MODE_NUNBER}'")
		
		set(VECTORIAL_SIZE_INT "1")
		set(VECTORIAL_SIZE_FLOAT "1")
		
		set(VECTORIAL_TYPE_FLOAT "float")
		set(VECTORIAL_TYPE_INT "int")
		
		if("${MODE_NUNBER}" EQUAL "0")
			file(APPEND ${OUTPUT_FILE} "//Find no vectorization\n#ifndef NOVECTORIZATION\n#define NOVECTORIZATION\n#endif\n")
			set(PLIB_VECTOR_SIZE_FLOAT "1")
			set(PLIB_VECTOR_SIZE_DOUBLE "1")
			set(PLIB_VECTOR_SIZE_SHORT "1")
			set(PLIB_VECTOR_SIZE_INT "1")
			set(PLIB_VECTOR_SIZE_LONG_INT "1")
		else()
			if(${MODE_NUNBER} GREATER_EQUAL 1)
				file(APPEND ${OUTPUT_FILE} "//Find SSSE3\n#ifndef PLIB_SSSE3\n#define PLIB_SSSE3 1\n#endif\n")
				set(VECTORIAL_SIZE_INT "16")
				set(VECTORIAL_SIZE_FLOAT "16")
				set(VECTORIAL_TYPE_FLOAT "__m128")
				set(VECTORIAL_TYPE_INT "__m128i")
			endif()

			if("${MODE_NUNBER}" GREATER_EQUAL "2")
				file(APPEND ${OUTPUT_FILE} "//Find SSE4.1\n#ifndef PLIB_SSE4_1\n#define PLIB_SSE4_1 1\n#endif\n")
				set(VECTORIAL_SIZE_FLOAT "16")
				set(VECTORIAL_TYPE_FLOAT "__m128")
			endif()
			
			if("${MODE_NUNBER}" GREATER_EQUAL "3")
				file(APPEND ${OUTPUT_FILE} "//Find SSE4.2\n#ifndef PLIB_SSE4_2\n#define PLIB_SSE4_2 1\n#endif\n")
				file(APPEND ${OUTPUT_FILE} "//Find SSE4\n#ifndef PLIB_SSE4\n#define PLIB_SSE4 1\n#endif\n")
				set(VECTORIAL_SIZE_INT "16")
				set(VECTORIAL_TYPE_INT "__m128i")
			endif()

			if(${MODE_NUNBER} GREATER_EQUAL 4)
				file(APPEND ${OUTPUT_FILE} "//Find AVX\n\#ifndef PLIB_AVX\n#define PLIB_AVX 1\n#endif\n")
				set(VECTORIAL_SIZE_FLOAT "32")
				set(VECTORIAL_TYPE_FLOAT "__m256")
			endif()

			if("${MODE_NUNBER}" GREATER_EQUAL "5")
				file(APPEND ${OUTPUT_FILE} "//Find AVX2\n#ifndef PLIB_AVX2\n#define PLIB_AVX2 1\n#endif\n")
				set(VECTORIAL_SIZE_INT "32")
				set(VECTORIAL_TYPE_INT "__m256i")
			endif()
			
			if("${MODE_NUNBER}" GREATER_EQUAL "6")
				file(APPEND ${OUTPUT_FILE} "//Find AVX512\n#ifndef PLIB_AVX512F\n#define PLIB_AVX512F 1\n#endif\n")
				set(VECTORIAL_SIZE_FLOAT "64")
				set(VECTORIAL_TYPE_FLOAT "__m512")
			endif()
			
			if("${MODE_NUNBER}" GREATER_EQUAL "7")
				file(APPEND ${OUTPUT_FILE} "//Find AVX512ER\n#ifndef PLIB_AVX512ER\n#define PLIB_AVX512ER 1\n#endif\n")
				set(VECTORIAL_SIZE_FLOAT "64")
				set(VECTORIAL_TYPE_FLOAT "__m512")
			endif()
			
			if("${MODE_NUNBER}" GREATER_EQUAL "8")
				file(APPEND ${OUTPUT_FILE} "//Find AVX512PF\n#ifndef PLIB_AVX512PF\n#define PLIB_AVX512PF 1\n#endif\n")
				set(VECTORIAL_SIZE_FLOAT "64")
				set(VECTORIAL_TYPE_FLOAT "__m512")
			endif()
			
			if("${MODE_NUNBER}" GREATER_EQUAL "9")
				file(APPEND ${OUTPUT_FILE} "//Find AVX512BW\n#ifndef PLIB_AVX512BW\n#define PLIB_AVX512BW 1\n#endif\n")
				set(VECTORIAL_SIZE_FLOAT "64")
				set(VECTORIAL_TYPE_FLOAT "__m512")
				set(VECTORIAL_SIZE_INT "64")
				set(VECTORIAL_TYPE_INT "__m512i")
			endif()
			
			if("${MODE_NUNBER}" GREATER_EQUAL "10")
				file(APPEND ${OUTPUT_FILE} "//Find AVX512\n#ifndef PLIB_AVX512_2\n#define PLIB_AVX512_2 1\n#endif\n")
				file(APPEND ${OUTPUT_FILE} "//Find AVX512\n#ifndef PLIB_AVX512\n#define PLIB_AVX512 1\n#endif\n")
				set(VECTORIAL_SIZE_INT "64")
				set(VECTORIAL_TYPE_INT "__m512i")
			endif()
		
			math(EXPR PLIB_VECTOR_SIZE_FLOAT "${VECTORIAL_SIZE_FLOAT}/ 4")
			math(EXPR PLIB_VECTOR_SIZE_DOUBLE "${VECTORIAL_SIZE_FLOAT}/ 8")
			
			math(EXPR PLIB_VECTOR_SIZE_SHORT "${VECTORIAL_SIZE_INT}/ 2")
			math(EXPR PLIB_VECTOR_SIZE_INT "${VECTORIAL_SIZE_INT}/ 4")
			math(EXPR PLIB_VECTOR_SIZE_LONG_INT "${VECTORIAL_SIZE_INT}/ 8")
		endif()
		
		file(APPEND ${OUTPUT_FILE} "#ifndef PLIB_VECTOR_SIZE_BYTE_FLOAT\n")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_VECTOR_SIZE_BYTE_FLOAT ${VECTORIAL_SIZE_FLOAT}\n")
		file(APPEND ${OUTPUT_FILE} "#endif\n\n")
		
		file(APPEND ${OUTPUT_FILE} "#ifndef PLIB_VECTOR_SIZE_BYTE_INT\n")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_VECTOR_SIZE_BYTE_INT ${VECTORIAL_SIZE_INT}\n")
		file(APPEND ${OUTPUT_FILE} "#endif\n\n")
		
		file(APPEND ${OUTPUT_FILE} "#ifndef PLIB_VECTOR_SIZE_FLOAT\n")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_VECTOR_SIZE_FLOAT ${PLIB_VECTOR_SIZE_FLOAT}\n")
		file(APPEND ${OUTPUT_FILE} "#endif\n\n")
		
		file(APPEND ${OUTPUT_FILE} "#ifndef PLIB_VECTOR_SIZE_DOUBLE\n")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_VECTOR_SIZE_DOUBLE ${PLIB_VECTOR_SIZE_DOUBLE}\n")
		file(APPEND ${OUTPUT_FILE} "#endif\n\n")
		
		file(APPEND ${OUTPUT_FILE} "#ifndef PLIB_VECTOR_SIZE_CHAR\n")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_VECTOR_SIZE_CHAR ${VECTORIAL_SIZE_INT}\n")
		file(APPEND ${OUTPUT_FILE} "#endif\n\n")
		
		file(APPEND ${OUTPUT_FILE} "#ifndef PLIB_VECTOR_SIZE_SHORT\n")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_VECTOR_SIZE_SHORT ${PLIB_VECTOR_SIZE_SHORT}\n")
		file(APPEND ${OUTPUT_FILE} "#endif\n\n")
		
		file(APPEND ${OUTPUT_FILE} "#ifndef PLIB_VECTOR_SIZE_INT\n")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_VECTOR_SIZE_INT ${PLIB_VECTOR_SIZE_INT}\n")
		file(APPEND ${OUTPUT_FILE} "#endif\n\n")
		
		file(APPEND ${OUTPUT_FILE} "#ifndef PLIB_VECTOR_SIZE_LONG_INT\n")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_VECTOR_SIZE_LONG_INT ${PLIB_VECTOR_SIZE_LONG_INT}\n")
		file(APPEND ${OUTPUT_FILE} "#endif\n\n")
		
		file(APPEND ${OUTPUT_FILE} "\n\n")
		file(APPEND ${OUTPUT_FILE} "typedef ${VECTORIAL_TYPE_FLOAT} PRegVecf;\n")
		file(APPEND ${OUTPUT_FILE} "typedef ${VECTORIAL_TYPE_INT} PRegVecI;\n")
		file(APPEND ${OUTPUT_FILE} "\n\n")
		file(APPEND ${OUTPUT_FILE} "\n")
		file(APPEND ${OUTPUT_FILE} "#endif\n")
		file(APPEND ${OUTPUT_FILE} "\n")
	endif()
endfunction(plib_core_createDefInclude)

# Get the flag to be provided to the compiler
# 	outputFlag : variable to be set
function(plib_core_getArchExtension outputFlag)
	plib_core_getVectMode(VECT_MODE)
	
	if(VECT_MODE STREQUAL "none")
		set(${outputFlag} "" PARENT_SCOPE)
	else()
		set(${outputFlag} "-m${VECT_MODE}" PARENT_SCOPE)
	endif()
endfunction(plib_core_getArchExtension)

# Create the include of the system caracteristics
# 	outputFileName : name of the file to be created
function(plib_core_findSystemCaracteristic outputFileName)
	get_filename_component(baseFileName ${outputFileName} NAME)
	set(OUTPUT_FILE "${CMAKE_CURRENT_BINARY_DIR}/${baseFileName}")
	if(EXISTS ${OUTPUT_FILE})
		message(STATUS "File '${OUTPUT_FILE}' already exists")
	else()
		string(REPLACE "." "_" lowDefine ${baseFileName})
		string(TOUPPER ${lowDefine} upperDefine)
		
		if(APPLE)
			execute_process(COMMAND bash "-c" "sysctl hw | grep l1dcachesize  | cut -s -d ' ' -f 2"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE CACHE_L1_DATA)
			execute_process(COMMAND bash "-c" "sysctl hw | grep l1icachesize  | cut -s -d ' ' -f 2"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE CACHE_L1_INSTRUCTION)
			execute_process(COMMAND bash "-c" "sysctl l2cachesize | grep L2 | cut -s -d ' ' -f 2"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE CACHE_L2)
			execute_process(COMMAND bash "-c" "sysctl l3cachesize | grep L3 | cut -s -d ' ' -f 2"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE CACHE_L3)
			execute_process(COMMAND bash "-c" "sysctl hw | grep \"byteorder\" | cut -d ' ' -f 2"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE ENDIANESS)
			execute_process(COMMAND bash "-c" "sysctl -n hw.ncpu"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE NB_CORE)
		else()
			execute_process(COMMAND bash "-c" "lscpu -B --cache=NAME,ONE-SIZE | grep L1d | sed -e \"s/ //g\" | sed -e \"s/L1d//g\" | sed -e s/K//g;"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE CACHE_L1_DATA)
			execute_process(COMMAND bash "-c" "lscpu -B --cache=NAME,ONE-SIZE | grep L1i | sed -e \"s/ //g\" | sed -e \"s/L1i//g\" | sed -e s/K//g;"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE CACHE_L1_INSTRUCTION)
			execute_process(COMMAND bash "-c" "lscpu -B --cache=NAME,ONE-SIZE | grep L2 | sed -e \"s/ //g\" | sed -e \"s/L2//g\" | sed -e s/K//g;"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE CACHE_L2)
			execute_process(COMMAND bash "-c" "lscpu -B --cache=NAME,ONE-SIZE | grep L3 | sed -e \"s/ //g\" | sed -e \"s/L3//g\" | sed -e s/M//g;"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE CACHE_L3)
			execute_process(COMMAND bash "-c" "lscpu | grep \"Byte Order\" | sed -e \"s/\ //g\" | cut -d ':' -f 2;"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE ENDIANESS)
			execute_process(COMMAND bash "-c" "cat /proc/cpuinfo | grep \"cpu cores\" | head -n 1 | cut -s -d ':' -f 2 | sed -e \"s/ //g\";"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE NB_CORE)
			execute_process(COMMAND bash "-c" "cat /proc/cpuinfo | grep \"model name\" | head -n 1 | cut -s -d ':' -f 2"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE CPU_MODEL_NAME)
			execute_process(COMMAND bash "-c" "cat /proc/cpuinfo | grep siblings | head -n 1 | cut -s -d ':' -f 2"
				WORKING_DIRECTORY "${CMAKE_CURRENT_BINARY_DIR}" OUTPUT_VARIABLE CPU_SIBLINGS)
		endif()
		
		if(CACHE_L1_DATA STREQUAL "")
			set(CACHE_L1_DATA "0")
		else()
			string(REPLACE "\n" "" CACHE_L1_DATA ${CACHE_L1_DATA})
		endif()
		
		if(CACHE_L1_INSTRUCTION STREQUAL "")
			set(CACHE_L1_INSTRUCTION "0")
		else()
			string(REPLACE "\n" "" CACHE_L1_INSTRUCTION ${CACHE_L1_INSTRUCTION})
		endif()
		
		if(CACHE_L2 STREQUAL "")
			set(CACHE_L2 "0")
		else()
			string(REPLACE "\n" "" CACHE_L2 ${CACHE_L2})
		endif()

		if(CACHE_L3 STREQUAL "")
			set(CACHE_L3 "0")
		else()
			string(REPLACE "\n" "" CACHE_L3 "${CACHE_L3}")
		endif()
		string(REPLACE "\n" "" NB_CORE ${NB_CORE})
		
		if(CPU_MODEL_NAME STREQUAL "")
			set(CPU_MODEL_NAME "0")
		else()
			string(REPLACE "\n" "" CPU_MODEL_NAME "${CPU_MODEL_NAME}")
		endif()
		
		if(CPU_SIBLINGS STREQUAL "")
			set(CPU_SIBLINGS "0")
		else()
			string(REPLACE "\n" "" CPU_SIBLINGS "${CPU_SIBLINGS}")
		endif()
		
		file(WRITE ${OUTPUT_FILE} "\n")
		file(APPEND ${OUTPUT_FILE} "/***************************************\n")
		file(APPEND ${OUTPUT_FILE} "	Auteur : Pierre Aubert\n")
		file(APPEND ${OUTPUT_FILE} "	Mail : aubertp7@gmail.com\n")
		file(APPEND ${OUTPUT_FILE} "	Licence : CeCILL-C\n")
		file(APPEND ${OUTPUT_FILE} "****************************************/\n")
		file(APPEND ${OUTPUT_FILE} "\n")
		file(APPEND ${OUTPUT_FILE} "#ifndef __${upperDefine}__\n")
		file(APPEND ${OUTPUT_FILE} "#define __${upperDefine}__\n")
		file(APPEND ${OUTPUT_FILE} "\n")
		
		file(APPEND ${OUTPUT_FILE} "///Number of core on the system\n")
		message(STATUS "CPU_MODEL_NAME = ${CPU_MODEL_NAME}")
		message(STATUS "CPU_SIBLINGS = ${CPU_SIBLINGS}")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_SYSTEM_NB_CORE ${NB_CORE}\n")
		message(STATUS "NB_CORE = ${NB_CORE}")
		file(APPEND ${OUTPUT_FILE} "\n")
		file(APPEND ${OUTPUT_FILE} "///Size of the L1 cache for data\n")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_CACHE_L1_DATA ${CACHE_L1_DATA}lu\n")
		message(STATUS "CACHE_L1_DATA = ${CACHE_L1_DATA} B")
		file(APPEND ${OUTPUT_FILE} "\n")
		file(APPEND ${OUTPUT_FILE} "///Size of the L1 cache for instruction\n")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_CACHE_L1_INTSRUCTION ${CACHE_L1_INSTRUCTION}lu\n")
		message(STATUS "CACHE_L1_INSTRUCTION = ${CACHE_L1_INSTRUCTION} B")
		file(APPEND ${OUTPUT_FILE} "\n")
		file(APPEND ${OUTPUT_FILE} "///Size of the L2 cache\n")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_CACHE_L2 ${CACHE_L2}lu\n")
		message(STATUS "CACHE_L2 = ${CACHE_L2} B")
		file(APPEND ${OUTPUT_FILE} "\n")
		file(APPEND ${OUTPUT_FILE} "///Size of the L3 cache\n")
		file(APPEND ${OUTPUT_FILE} "#define PLIB_CACHE_L3 ${CACHE_L3}lu\n")
		message(STATUS "CACHE_L3 = ${CACHE_L3} B")
		file(APPEND ${OUTPUT_FILE} "\n")
		file(APPEND ${OUTPUT_FILE} "///Endianess of the system\n")
		file(APPEND ${OUTPUT_FILE} "/**	LittleEndian or BigEndian\n")
		file(APPEND ${OUTPUT_FILE} "*/\n")
		if(APPLE)
			if(${ENDIANESS} EQUAL "1234")
				file(APPEND ${OUTPUT_FILE} "#define PLIB_ENDIANESS LittleEndian\n")
				message(STATUS "ENDIANESS = LittleEndian")
			else()
				file(APPEND ${OUTPUT_FILE} "#define PLIB_ENDIANESS ${ENDIANESS}\n")
				message(STATUS "ENDIANESS = ${ENDIANESS}")
			endif()
		else()
			if(ENDIANESS)
				file(APPEND ${OUTPUT_FILE} "#define PLIB_ENDIANESS ${ENDIANESS}\n")
				message(STATUS "ENDIANESS = ${ENDIANESS}")
			else()
				file(APPEND ${OUTPUT_FILE} "#define PLIB_ENDIANESS LittleEndian\n")
				message(STATUS "ENDIANESS = LittleEndian")
			endif()
		endif()
		
		file(APPEND ${OUTPUT_FILE} "\n")
		file(APPEND ${OUTPUT_FILE} "#endif\n")
		file(APPEND ${OUTPUT_FILE} "\n")
	endif()
endfunction(plib_core_findSystemCaracteristic)


# Create the compilation flag for vectorization, phoenix_architecture_def.h and phoenix_system_def.h files
# 	create also the global variable PHOENIX_FLAG_VECTORIZED_COMPILATION which contains vectorization flags for compilation
function(phoenix_architecture)
	if(TARGET phoenix_architecture_target)
		message(STATUS "Architecture already get, target phoenix_architecture_target already exists")
	else()
		add_custom_target(phoenix_architecture_target)
		plib_core_getArchExtension(FLAG_VECTORIZED_COMPILATION)
		message(STATUS "tmp FLAG_VECTORIZED_COMPILATION = ${FLAG_VECTORIZED_COMPILATION}")
		set(PHOENIX_FLAG_VECTORIZED_COMPILATION "${FLAG_VECTORIZED_COMPILATION}" CACHE STRING "vectorized compilation flag")
		message(STATUS "global PHOENIX_FLAG_VECTORIZED_COMPILATION = ${PHOENIX_FLAG_VECTORIZED_COMPILATION}")
		plib_core_createDefInclude(phoenix_architecture_def.h)
		plib_core_findSystemCaracteristic(phoenix_system_def.h)
	endif()
endfunction(phoenix_architecture)



